/* -*-C++-*- */
/*
 *  Copyright (c) 1999 Regents of the University of California.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgment:
 *       This product includes software developed by the Imaging and
 *       Distributed Collaboratories Group at Lawrence Berkeley
 *       National Laboratory.
 *  4. Neither the name of the University nor of the Laboratory may be used
 *     to endorse or promote products derived from this software without
 *     specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 */


#ifndef _ACL_AUTHORIZER_H_
#define _ACL_AUTHORIZER_H_

//--------------------------------------------------------------------
// A sub-class of SimpleACLAuthorizer, it uses a more complicated acl file
// that contains both names and rights.  The acl file should be formated
// so that lines containing names begin with the keyword "name" followed
// by the principal name.  A newline, and the the keyword "rights" followed
// by a comma seperated list of rights.  For example:
//                   name kjackson
//                   rights read, write, execute
//                   name deba
//                   rights read, write, execute
// lines beginning with a "#" are considered comments.
//----------------------------------------------------------------------
#include "SimpleACLAuthorizer.h"
#include <string>

class ACLAuthorizer : public SimpleACLAuthorizer
{
  public:
//---------------------------------------------------------------
// The constructors take a single argument.  It is a string containing
// the full path to the acl file.  A string will be thrown if unable to
// open the file, or if the file is corrupted in some way.
//-----------------------------------------------------------------
    ACLAuthorizer(const std::string& file);
    ACLAuthorizer(const char* file);
    virtual ~ACLAuthorizer( void );
  protected:
    virtual bool checkAction(std::string& rqstAct, std::string& actList);
  private:
    void readACLFile(const char* file);
    void parseEntry(std::string& name, std::string& rights);
};

#endif //_ACL_AUTHORIZER_H_
