/* -*-C++-*- */
/*
 *  Copyright (c) 1999 Regents of the University of California.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgment:
 *       This product includes software developed by the Imaging and
 *       Distributed Collaboratories Group at Lawrence Berkeley
 *       National Laboratory.
 *  4. Neither the name of the University nor of the Laboratory may be used
 *     to endorse or promote products derived from this software without
 *     specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 */

#include <cstdio>
#include <cstring>
#include <cstdlib>
#include "MyDebug.h"
#include <iostream>

using std::cout;
using std::endl;
using std::printf;
using std::vprintf;
using std::string;


MyDebug::MyDebug( const string name )
    : mPrintall(5), mClassname(name)
{
}

MyDebug::~MyDebug()
{
}

/*  printDebug - allows the debugging printouts to be filtered
 *  according the the current debug level setting.  The first
 *  argument is what debug level this should be printed at.
 *  It is printed if the current debug level is at this level
 *  or higher.  THe rest of the arguments are the formatting string
 *  and arguments for a printfd statement.
 */

void MyDebug::printDebug( int level, const char *fmt, ... )
{
    va_list ap;

    if( mDebugLevel >= level )
        {
	cout << mClassname << ": ";
        va_start(ap, fmt );
        vprintf( fmt, ap );
	va_end( ap );
	}
    return;
}

void MyDebug::printDebug(int level, const string msg)
{
    if (mDebugLevel >= level)
    {
	cout << mClassname << ": " << msg << endl;
    }   
}
/*  printExactDebug - allows the debugging printouts to be filtered
 *  according the the current debug level setting.  The first
 *  argument is what debug level this should be printed at.
 *  It is printed if the current debug level is at this level
 *  or if debug level is greater than or equal to the level that indicates
 *  that all debugging info should be printed.  THe rest of 
 *  the arguments are the formatting string
 *  and arguments for a printf statement.
 */

void MyDebug::printExactDebug( int level, const char *fmt, ... )
{
    va_list ap;

    if( mDebugLevel == level || mDebugLevel >= mPrintall )
        {
	    cout << mClassname << ": ";
        va_start(ap, fmt );
        vprintf( fmt, ap );
	va_end( ap );
	}
    return;
}

void MyDebug::printExactDebug( int level, const string msg)
{
    if (mDebugLevel == level || mDebugLevel >= mPrintall)
    {
	cout << mClassname << ": " << msg << endl;
    }
}

/*  printError - this function is called to print out
 *  errors that are really more like warnings.  They are
 *  only printed out if the current debug level is greater
 *  than zero.
 */
void MyDebug::printError( const char *fmt, ... )
{
    va_list ap;
    if( mDebugLevel >= 1 )
        {
	    cout << "ERROR - " << mClassname << ": ";
        va_start(ap, fmt );
        vprintf( fmt, ap );
	va_end( ap );
	}
    return;
}

void MyDebug::printError( const string msg)
{
    if (mDebugLevel >= 1)
    {
	cout << "ERROR - " << mClassname << ": " << msg << endl;
    }
}
