/* -*-C++-*- */
/*
 *  Copyright (c) 1999 Regents of the University of California.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgment:
 *       This product includes software developed by the Imaging and
 *       Distributed Collaboratories Group at Lawrence Berkeley
 *       National Laboratory.
 *  4. Neither the name of the University nor of the Laboratory may be used
 *     to endorse or promote products derived from this software without
 *     specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 */


/*
 *  Methods defining the Thread class
 */

#include "MyThread.h"
#include <stdio.h>

extern "C"
{
    void* mythread_start_func(void* arg)
    {
	return MyThread::CallMain(arg);
    }
}

MyThread::MyThread( )
{
    // initialize any variables
    thread_id = 0;
    
}

MyThread::~MyThread()
{
    // stop the thread's execution
}

void MyThread::StartThread()
{
    printf("MyThread::StartThread()\n");
    int rtnval;
    // now we can start the thread running so call pthread_create and have
    // it start running ThreadMain.
    pthread_attr_t attr;
    pthread_attr_init(&attr);
    pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
    rtnval = pthread_create( &thread_id, &attr, mythread_start_func, (void *)this );

    if( rtnval != 0 ){
	printf("Unable to create pthread rtnval = %d\n", rtnval);
	return;
    }
}

void MyThread::Begin()
{
    // complete initialization to get the thread running

    //  if the thread_id is not zero then we have already started a
    //  thread and not detached it.  Detach the previous thread before
    //  creating the new thread.

    printf("MyThread::Begin()" );
/*
    if( thread_id != 0 )
        {
	printf( "Current thread being detached before starting new thread\n");
	pthread_detach( thread_id );
	thread_id = 0;
	}
*/
    StartThread();
    return;
}

void MyThread::Cancel()
{
    pthread_t  temp_thread_id;
    // call the cleanup routine to give the thread a chance to exit cleanly
    // for some reason this call is not executing correctly
    Cleanup();
    // cancel the currently running thread.
    temp_thread_id = thread_id;
    thread_id = 0;

    pthread_cancel( temp_thread_id );
}

void MyThread::Join()
{
  //  wait for thread to complete
    pthread_join( thread_id, NULL );
    thread_id = 0;
}

void MyThread::Detach()
{
    //detach the running thread
    int err;
    if( thread_id > 0 )
      err = pthread_detach( thread_id );

    thread_id = 0;
    printf("MyThread::Detach() detach Thread err=%d\n", err);
}

void MyThread::Cleanup()
{
   printf("MyThread::Cleanup\n");
   pthread_exit(NULL);
   thread_id = 0;
}

void *MyThread::CallMain( void *objptr )
{
    MyThread *thisptr = (MyThread *)objptr;
    // call the ThreadMain routine
    thisptr->ThreadMain();
    return NULL;
}
