/* -*-C++-*- */
/*
 *  Copyright (c) 2000 Regents of the University of California.
 *  All rights reserved.
 * 
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgment:
 *       This product includes software developed by the Imaging and
 *       Distributed Collaboratories Group at Lawrence Berkeley 
 *       National Laboratory.
 *  4. Neither the name of the University nor of the Laboratory may be used
 *     to endorse or promote products derived from this software without
 *     specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 * Virtual base class for results from various tests
 */

#ifndef RESULTS_H
#define RESULTS_H

#include <iosfwd>
#include <iostream>
#include <string>
#include <map>
#include <sys/types.h> //pthread_t -> unsigned int
#include "config.h"
#include "MyDebug.h"
#include "headers.h"
#include "../nettest/CoordInfo.h"

class PingResults;
class IperfResults;
class PerfSocket;
class PingTestInfo;

using std::string;
using std::map;
typedef multimap < pthread_t, IperfResults* > IPMap;
typedef multimap < pthread_t, IperfResults* >::iterator IPMapi;


class Results: public MyDebug {

protected:

   bool isNLping;
   bool isNLiperf;
   bool isNLSO;
   bool isPrintPing;
   bool isPrintIperf;
   bool isPrintSO;
   bool isNLenabled;

   bool isReportEndOfTest;
   bool isFinalReport;

   // multimap hold iperfResults
   IPMap ip_Map;


#if USE_NETLOGGER
   struct _loginfo * Nlh;
   string _netloghost;
#endif

public:

   Results( void );
#if USE_NETLOGGER
   Results( string );
   void RefreshNetLoggerHandle( void );
#endif
   ~Results( void );

   int  setNetLogHost( string );
   void newPingResults( PingResults );
   void iperfResults( PerfSocket&, const double, const double, const max_size_t );
   void iperfResults( IperfResults& );
   int iperfResults_EndOfTest( pthread_t );

   void iperfResults_Jitter( PerfSocket&, const double, const double, const int32_t, 
			     const int32_t, const int32_t, const max_size_t );
   void iperfResults_Jitter( IperfResults& );
   int iperfResults_Jitter_EndOfTest( pthread_t );

   void tracerouteResults( bool, string, string, string );

   void pipecharResults( bool, CoordInfo&, string );

   inline void setIsFinalReport( bool isFR )
   {
	isFinalReport = isFR;
   };
   inline void setIsReportEndOfTest( bool isREOF )
   {
	isReportEndOfTest = isREOF;
   };
   inline void setIsNLenabled( bool isNL )
   {
	isNLenabled = isNL;
   };
   inline void setIsNLping( bool isPing )
   {
	isNLping = isPing;
   };
   inline void setIsNLiperf( bool isIperf )
   {
	isNLiperf = isIperf;
   };
   inline void setIsNLSO( bool isSO )
   {
	isNLSO = isSO;
   };
   inline void setIsPrintPing( bool isPing )
   {
	isPrintPing = isPing;
   }; 
   inline void setIsPrintIperf( bool isIperf )
   {
	isPrintIperf = isIperf;
   };
   inline void setIsPrintSO( bool isSO )
   {
	isPrintSO = isSO;
   };
   
   inline bool getIsFinalReport( void )
   {
	return isFinalReport;
   };
   inline bool getIsReportEndOfTest( void )
   {
	return isReportEndOfTest;
   };
   inline bool getIsNLenabled( void )
   {
	return isNLenabled;
   };
   inline bool getIsNLping( void )
   {
	return isNLping;
   };
   inline bool getIsNLiperf( void )
   {
	return isNLiperf;
   };
   inline bool getIsNLSO( void )
   {
	return isNLSO;
   };
   inline bool getIsPrintPing( void )
   {
	return isPrintPing;
   };
   inline bool getIsPrintIperf( void )
   {
	return isPrintIperf;
   };
   inline bool getIsPrintSO( void )
   {
	return isPrintSO;
   };
};

#endif
