/* -------------------------------------------------------------------------
 *  Copyright (c) 1999 Regents of the University of California.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgment:
 *       This product includes software developed by the Imaging and
 *       Distributed Collaboratories Group at Lawrence Berkeley
 *       National Laboratory.
 *  4. Neither the name of the University nor of the Laboratory may be used
 *     to endorse or promote products derived from this software without
 *     specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 *  $Id: lblnettest.cc,v 1.68 2001/09/10 16:40:05 boverhof Exp $
 * -------------------------------------------------------------------------*/

#include "lblnettest.h"
#include <iostream>
//#include <wait.h>
#include "Display_funcs.h"
#include "funcs.h"
#include "Results.h"
//#include "TestInfo.h"
#include "CoordInfo.h"
#include "NetworkTcp.h"
#include "MyDebug.h"
//#include "config.h"
#include "TestModule.h"

using std::cout;
using std::endl;

// defined in Display_funcs.h
extern const string REPORTS;
extern const string MASTER;
extern const string IPERF;
extern const string UDPPING;
extern const string MULTIPING;
extern const string TUNEDTCP;
extern const string TRACEROUTE;
extern const string QUIT;
extern const string EMPTY;
extern const string FAILED;
extern const string FINISHED;
extern const string DELIM;
extern const string IPERF_PORT;
extern const string UPDDING_PORT;
extern const string DEBUG;

// Debugging & Integers Used with MyDebug
const int d1 = 4;
const int d2 = 2;

string debugLevel_Curtest; //DDD
DisplayFunc display;
TestModule tModule;
MyDebug debug("Main");
sigset_t signals;  // needed for sigmask()

int
main(int argc, char *argv[])
{
    int        err;
    NetworkTCP network_securetcp;
    Results    results;
    SecureTcp *relaycoordcon = NULL;

    cerr << "Nettest " << __DATE__ << " " << __TIME__ << endl;

    setSignals_to_Handle();
    // Required by sigmask().
    sigemptyset(&signals);
    sigfillset(&signals);
    signal(SIGINT, sighandler);
    signal(SIGTERM, sighandler);
    signal(SIGABRT, sighandler);
    signal(SIGSEGV, sighandler);
#ifdef SIGBUS
    signal(SIGBUS, sighandler);
#endif
#ifdef SIGHUP
    signal(SIGHUP, sighandler);
#endif
#ifdef SIGQUIT
    signal(SIGQUIT, sighandler);
#endif
#ifdef SIGPIPE
    signal(SIGPIPE, sighandler);
#endif

    string localhost( getLocalHostName() );
    if( localhost.empty() )
    {
	cout << "**Couldn't Determine Local Host Name\n";
	exit(-1);
    }

    CoordInfo coordinfo( *doCommandLineRequest(&results, network_securetcp, 
						localhost, argc, argv) );

    string  curtest; 

    while( 1 )
    {
        /* ---------------------------------------------------------------------
         *     Wait for command or input then proceed to Individual tests
	 *
         * ---------------------------------------------------------------------*/

        if( coordinfo.getIsTestRequester() ) //REQUESTER
	{
	    if( relaycoordcon != NULL )
		delete relaycoordcon;

            err = Requester_GetNextTest( curtest, coordinfo ); //fill coordinfo
	    if( err == 1 ){
            	if( coordinfo.getIsTestMaster() ){ // REQUESTER-MASTER
                    debug.printDebug(d1, "#### MASTER: %s\n", curtest.c_str());
                    if( handleRequest_Master(curtest, coordinfo, network_securetcp, NULL) < 0 )
		    	cout << "Requester/Master/TEST FAILED" << endl;
		    else
		    	cout << "Requester/Master/TEST FINISHED" << endl;
            	}
	    	else{ 
		    coordinfo.setCoordTestHost( coordinfo.getMasterName() );
            	    relaycoordcon = network_securetcp.newTCP_connect( coordinfo.getCoordTestHost(), *passwdC_cb_Wrap );
		    if( relaycoordcon != NULL ){
	    	        coordConSend( relaycoordcon, curtest );
                    	debug.printDebug(d1, "Requester #1C - newTCP_connect\n");
		    	if( (coordinfo.getIsTestSender()||coordinfo.getIsTestReceiver()) ){
			    SecureTcp *reqTestCC;
 			    if( (  coordinfo.getIsSOTest()&&coordinfo.getIsTestSender() ) || 
			        ( !coordinfo.getIsSOTest()&& (coordinfo.getIsTestReceiver()||coordinfo.getIsTestSender()) ) ){

			    	reqTestCC = network_securetcp.newTCP_accept( *passwdC_cb_Wrap );
			    	if( curtest == coordConReceive(reqTestCC) ){
		    	            doTest( coordinfo );
	    	    	            coordConSend( reqTestCC, FINISHED );
			        }
			        else
	    	    	            coordConSend( reqTestCC, FAILED );
			    }
	    	    	    else
		    	    	debug.printDebug(d1, "Requester NOT sender/test_client/master\n");
		    	}
            	    	err = wait_for_completed_Test(relaycoordcon); // Master - FINISHED -> Req
		    } //NULL
	    	}
	    }
	    else if( err < 0 )
		cout << "ERROR: Test Request ignored\n";
	    else
	      debug.printDebug(d1, "Selected c-change master or o-change output\n");
        }
        else //SERVER
	{
            cout << "\n### Waiting for a connection\n";

	    string sender, client;
	    relaycoordcon = network_securetcp.newTCP_accept( *passwdS_cb_Wrap );
	    if( relaycoordcon == NULL ){ 
		debug.printDebug(2, "Start Over- SERVER relaycoordcon is NULL\n");
		continue;
	    }

            curtest = coordConReceive( relaycoordcon );
            debug.printDebug( d1, "Server Test Received: %s\n", curtest.c_str() );
	    if( curtest.empty() ){
            	debug.printDebug( d1, "Nothing Received\n");
		continue;
 	    }
	    else
                debug.printDebug(d1, "curtest not empty\n");

	    // set Name of this host sun(gethostbyname_r) else (gethostname)
	    localhost = relaycoordcon->getLocalName(); //securetcp -> tcpnet -> network
	    if (getCanonicalName( localhost ) < 0 ){
            	debug.printDebug( d1, "Cannot determine my Local name\n");
		continue;
 	    }

	    // **0th debugLevel 1/testName/master/sender/client
	    char* db = &(curtest[0]);
	    debug.setDebugLevel( atoi(db) );

	    if( debug.getDebugLevel() > 5 )      
		debug.setDebugLevel(5);
	    else if( debug.getDebugLevel() < 0 ) 
		debug.setDebugLevel(0);

            debug.printDebug( d1, "debugLevel set %d\n", debug.getDebugLevel() );

	    // **1st test
	    string testName = getToken( curtest, DELIM, 1 );
	    if(  check_TestName( testName ) < 0 ){
	        debug.printDebug(d1, "Start Over- Bad Test Name\n");
		continue;
	    }
	    else
	        debug.printDebug(d1, "1)test: %s\n", testName.c_str());

	    coordinfo.setTestName( testName );

	    ((testName == TRACEROUTE) || (testName == PIPECHAR))?
		coordinfo.setIsSOTest(true):
		coordinfo.setIsSOTest(false);

	    // **2nd master
	    ( localhost == getToken( curtest, DELIM, 2 ) )?
		coordinfo.setIsTestMaster( true):
		coordinfo.setIsTestMaster(false);

	    // **3rd sender
	    sender = getToken( curtest, DELIM, 3 );
	    if(  sender.empty() ){
	        debug.printDebug(d1, "No sender Specified\n");
		continue;
	    }
	    else
	        debug.printDebug(d1, "sender: %s\n", sender.c_str());
	    coordinfo.setTestSender( sender );

	    // **4th client
	    client = getToken( curtest, DELIM, 4 );
	    if(  client.empty() ){
	        debug.printDebug(d1, "No client Specified\n");
		continue;
	    }
	    else
	        debug.printDebug(d1, "4) Client: %s\n", client.c_str());
	    coordinfo.setTestReceiver( client );

	    // **5th Parameters
	    if(  setCoordInfoParam( curtest, coordinfo ) == false ){
	        debug.printDebug(d1, "What test params?\n");
		continue;
	    }
	    else
	        debug.printDebug(d1, "5) parameters");

	    coordinfo.setIsTestReceiver( (client==localhost)?true:false );
	    coordinfo.setIsTestSender(   (sender==localhost)?true:false );

	    if( coordinfo.getIsTestMaster() )
	    { //SERVER-MASTER
		debug.printDebug(d1, "#### MASTER: %s\n", curtest.c_str());
		handleRequest_Master(curtest,coordinfo,network_securetcp,relaycoordcon);
	    }
    	    else if( coordinfo.getIsTestReceiver() )
	    {
		debug.printDebug(d1, "#### CLIENT TEST SITE: %s\n", curtest.c_str());
		doTest( coordinfo );
                completed_Test( true, relaycoordcon ); // tell Master Done
	    }
    	    else if( coordinfo.getIsTestSender() )
	    {
		debug.printDebug(d1, "#### SERVER TEST SITE:  %s\n", curtest.c_str());
		doTest( coordinfo );
                completed_Test( true, relaycoordcon ); // tell Master Done
	    }
	    else
		cout << "#### ERROR: NOT A TEST_SITE/MASTER \n  curtest: " << curtest << endl;
	}
    } //while

    return 0;
}

/* -------------------------------------------------------------------
 * setSignals_to_Handle
 *
 * ------------------------------------------------------------------- */

void setSignals_to_Handle()
{
    // Required by sigmask().
    sigemptyset(&signals);
    sigfillset(&signals);
    signal(SIGINT, sighandler);
    signal(SIGTERM, sighandler);
    signal(SIGABRT, sighandler);
    signal(SIGSEGV, sighandler);
#ifdef SIGBUS
    signal(SIGBUS, sighandler);
#endif
#ifdef SIGHUP
    signal(SIGHUP, sighandler);
#endif
#ifdef SIGQUIT
    signal(SIGQUIT, sighandler);
#endif
#ifdef SIGPIPE
    signal(SIGPIPE, sighandler);
#endif
}

/* -------------------------------------------------------------------
 * doCommandLineRequest
 *  RET -1 fail
 *       0 daemon operation
 *      +1 normal operation
 *
 * ------------------------------------------------------------------- */

CoordInfo* 
doCommandLineRequest( Results *results_p, NetworkTCP &networkTcp, 
		      string localHost, int &argc, char *argv[] )
{

#if USE_NETLOGGER
    const char options[] = "D:dp:a:b:h:k:c:n:sp";
#else
    const char options[] = "D:dp:a:b:h:k:c:sp";
#endif
    debug.printDebug(d2, "doCommandLineRequest\n");
   
    bool isNotServerNorRequester(true); 
    bool notDaemon(true); 
    int c, i(1);

    CoordInfo *c_info_p;
    results_p->setIsNLenabled(false);

    // Parse command-line arguments using getopt
    c = getopt(argc, argv, options);
    do{
        // Check for the different command-line flags
        switch (c)
        {
            case 'D': // debug level
            {
                debug.printDebug(d2, "\tDebug level\n");
                int lvl = atoi(optarg);
                if ( lvl >= 0 && lvl <= 5 )
                {
                    debug.setDebugLevel(lvl);
		    debugLevel_Curtest = optarg; //DDD
                    cout << "Debug level set to " << debug.getDebugLevel() << endl;
              }
                else
                {
                    cout << "Debug level '" << optarg << "' not recognized" << endl;
                    display.displayUseage();
                    exit(-1);
                }
                break;
            }
            case 'h': // MASTER_NAME
            {
                debug.printDebug(d2, "\tRequester\n");

		if( !isNotServerNorRequester ){
                    display.displayUseage();
		    exit(-1);
		}
    		isNotServerNorRequester = false; 
		c_info_p = new CoordInfo(true, localHost);

                string master_host = string(optarg);
                if( getCanonicalName(master_host) < 0 ){
		    cout << "Cannot Determine " << master_host << "'s canonical name\n";
		    exit(-1);
 		}
		( master_host == localHost )?
                    c_info_p->setIsTestMaster( true ):
                    c_info_p->setIsTestMaster( false);

                c_info_p->setMasterName( master_host ); 
 		c_info_p->setRequesterName( localHost );
                i++;
                break;
            }
            case 's': // THIS_IS_A_SERVER
            {
                debug.printDebug(d2, "\tServer Master\n");
		if( !isNotServerNorRequester ){
                    display.displayUseage();
		    exit(-1);
		}
    		isNotServerNorRequester = false; 
		c_info_p = new CoordInfo(false, localHost);
                c_info_p->setIsTestMaster( false );
                break;
            }
            case 'k': // name of the key file
            {
                debug.printDebug(d2, "\tSetting key filename to %s\n", optarg);
                networkTcp.setClientKeyfilename( optarg );
                i++;
                break;
            }
            case 'c': // name of the certificate file
            {
                debug.printDebug(d2, "Setting Certificate file to %s\n", optarg);
                networkTcp.setClientCertfilename( optarg );
                i++;
                break;
            }
            case 'd': // Run as Daemon Need to use decrypted key
            {
                debug.printDebug(d2, "\tRun As Daemon\n");
                notDaemon = false;
                break;
            }
            case 'a':
            {
                debug.printDebug(d2, "\tclient side passphrase\n");
                temp_passwdC( string(optarg) );
                ++i;
                memset( (void*)argv[i], 'A', strlen(argv[i]) );
                break;
            }
            case 'b':
            {
                debug.printDebug(d2, "\tserver side passphrase\n");
                temp_passwdS( string( optarg ));
                ++i;
                memset( (void*)argv[i], 'B', strlen(argv[i]) );
                break;
            }
#if USE_NETLOGGER
            case 'n': // The Host running netlogd
            {
                debug.printDebug(d2, "\tnetlogd running on %s\n", optarg);
                string hostname( optarg );

                if( results_p == NULL ){
                    cout << "ERROR doCommandLine case n\n";
                    exit(-1);
                }
		else{
        	   results_p->setIsNLenabled(true);
                   results_p->setNetLogHost(hostname);
#ifdef PATH_TRACEROUTE
		   results_p->setIsNLSO(true);
		   results_p->setIsPrintSO(true);
#endif
#ifdef PATH_PIPECHAR
		   results_p->setIsNLSO(true);
		   results_p->setIsPrintSO(true);
#endif
		}

                i++;
                break;
            }
#endif
            default:
            {
                display.displayUseage();
                exit(-1);
            }
        }

        i++;

    } while((c = getopt(argc, argv, options)) != -1);

    if( isNotServerNorRequester )
    {
        display.displayUseage();
        exit(-1);
    }

    //If debugLevel_Curtest isn't specified set it to zero
    if( debugLevel_Curtest.empty() ){
        debugLevel_Curtest = "0";
        debug.setDebugLevel(0);
    }

    c_info_p->setResults_p( results_p );
    if( notDaemon ){
        if( setAsInterface(c_info_p, networkTcp) < 0 )
            exit(-1);
    }
    else{
        setAsDaemon( c_info_p );
        if( networkTcp.testCallBackFunction( false, *passwdC_cb) < 0 ){
            cout << "**Wrong Client Passphrase\n";
            exit(-1);
        }
        if( networkTcp.testCallBackFunction( true, *passwdS_cb) < 0 ){
            cout << "**Wrong Server Passphrase\n";
            exit(-1);
        }
    }
    return c_info_p;
}


/* ---------------------------------------------------------------------------------
 *  DAEMON OPERATION: redirect stdin to a file for passphrases.
 *  doCommandLine sets up
 *    1. Call temp_passwdS : set up server passwdS_cb function
 *            sockAccept (common/funcs.h)
 *    2. Call temp_passwdC : set up client passwdC_cb function
 *            sockConnect (common/funcs.h)
 * --------------------------------------------------------------------------------- */

void
setAsDaemon( CoordInfo *c_p )
{
        interface_Input( false ); // funcs.h
        c_p->getResults_p()->setIsPrintIperf( true );
        c_p->getResults_p()->setIsPrintPing( true );

#if USE_NETLOGGER
        if( (c_p->getResults_p())->getIsNLenabled() ){
            c_p->getResults_p()->setIsNLiperf( true );
            c_p->getResults_p()->setIsNLping( true );
        }
        else{
            c_p->getResults_p()->setIsNLiperf( false );
            c_p->getResults_p()->setIsNLping( false );
        }
#else
        c_p->getResults_p()->setIsNLiperf( false );
        c_p->getResults_p()->setIsNLping( false );
#endif
}


/* ---------------------------------------------------------------------------------
 *  USER INTERFACE OPERATION
 *  1. Call passwdS_cb : server passwd cb function sockAccept (common/funcs.h)
 *  2. Call passwdC_cb : client passwd cb function sockConnect (common/funcs.h)
 *
 *  3. relaycoordcon->setPassword_cb : set
 *  4.
 * --------------------------------------------------------------------------------- */

int
setAsInterface( CoordInfo *c_p, NetworkTCP &netTCP )
{
    interface_Input( true ); // funcs.h
    c_p->getResults_p()->setIsPrintIperf( true );
    c_p->getResults_p()->setIsPrintPing( true );

#if USE_NETLOGGER
    if( (c_p->getResults_p())->getIsNLenabled() ){
        c_p->getResults_p()->setIsNLping( true );
        c_p->getResults_p()->setIsNLiperf( true );
    }
    else{
        c_p->getResults_p()->setIsNLping( false );
        c_p->getResults_p()->setIsNLiperf( false );
     }
#else
    c_p->getResults_p()->setIsNLiperf( false );
    c_p->getResults_p()->setIsNLping( false );
#endif

    passwdC_cb( NULL, 0, 0, NULL );
    if( netTCP.testCallBackFunction( false, *passwdC_cb) < 0 ){
	cout << "**Wrong Passphrase\n";
	return -1;
    }

    passwdS_cb( NULL, 0, 0, NULL );
    if( netTCP.testCallBackFunction( true, *passwdS_cb) < 0 ){
	cout << "**Wrong Passphrase\n";
	return -1;
    }

    return 0;
}


/* ---------------------------------------------------------------------------------
 * setCoordInfoParam
 *
 * debug/test/master/server/client/ parameters
 *
 * --------------------------------------------------------------------------------- */

bool
setCoordInfoParam( string ctest, CoordInfo& c_info )
{
    bool   ret      = true;
    string testName = c_info.getTestName();

    string::size_type begIdx, endIdx;

    endIdx = ctest.length();
    begIdx = ctest.find_first_of( DELIM, 0 );
    begIdx = ctest.find_first_not_of( DELIM, begIdx );

    for( int i=0; i<4; i++ ){ // get to 4th token
        begIdx = ctest.find_first_of( DELIM, begIdx );
        begIdx = ctest.find_first_not_of( DELIM, begIdx );
    }

    string params;
    if( begIdx == string::npos )
        params = "";
    else 
	params.assign( ctest, begIdx, (endIdx-begIdx) );

    if( testName == IPERF )
        c_info.setIperfParams( params );
    else if( testName == UDPPING )
        c_info.setPingParams( params );
    else if( testName == MULTIPING )
        c_info.setMultiParams( params );
    else if( testName == TUNEDTCP )
        c_info.setTunedParams( params );
    else if( testName==TRACEROUTE )
	c_info.setTraceParams( params );
    else if( testName==PIPECHAR )
	c_info.setTraceParams( params );
    else
	 ret = false;
	
    return ret;
}


/* ---------------------------------------------------------------------------------
 *  check_TestName: If testname invalid return -1, else return 0
 *
 * --------------------------------------------------------------------------------- */

int
check_TestName( const string testName )
{
    if(  testName == IPERF     || testName == UDPPING     || testName == MULTIPING || 
	 testName == TUNEDTCP  || testName == TRACEROUTE  || testName == PIPECHAR   )
	return 0;
    else
       cout << "ERROR: test?? " << endl;

    return -1;
}


/* ---------------------------------------------------------------------------------
 * Requester_GetNextTest: Doesn't return unless valid request
 *	string    &curtest: Fill in test string to send
 *	CoordInfo &c_info :  parameters configured
 *
 *	Return 0: No Test to Perform
 *	       1: Perform Test
 *	      -1: Error
 *
 *      /DEBUG_INT/iperf/testMaster/testServer/testReceiver/parameters
 *
 * --------------------------------------------------------------------------------- */

int
Requester_GetNextTest( string &curtest, CoordInfo &c_info )
{
    debug.printDebug(d2, "Requester_GetNextTest\n");
    string testName;
    int fail;
    
    do{
	fail = 0;
        debug.printDebug(d2, "displayAskUser\n");

        curtest = debugLevel_Curtest; //global string
        curtest.append( DELIM );

	testName = display.displayAskUser();
	c_info.setTestName( testName );
        curtest.append( testName );
        curtest.append( DELIM );

        debug.printDebug(d1,"** Request for test '" + testName  + "'\n");

        if( testName==REPORTS ){
            display.displayChangedReportResults( c_info.getResults_p() );
	    return 0;
	}
	else if( testName==DEBUG ){
            display.changeDebugLevel( debugLevel_Curtest );
	    return 0;
	}
	else if( testName==MASTER ){
	    string master = c_info.getMasterName();
            display.getMaster(master);

	    if( !master.empty() ){
    	    	if( getCanonicalName( master ) < 0 ){
	            cout << "Master canonical name failed: " << master << endl;
		    return -1;
                }
	        else{
	   	    (c_info.getLocalHost() == master)?
		    	c_info.setIsTestMaster( true ):
		    	c_info.setIsTestMaster( false );
		    c_info.setMasterName( master );
	    	}
	    }
	    return 0;
	}
        else if( testName!="" && testName!=QUIT ){ // TESTS
            int no_defaults = 1;   // prints default test hosts to stdout
	   
            // Message Always begin with : testname/master/ 
            curtest.append( c_info.getMasterName() ); 
            curtest.append( DELIM );

	    //SO: testname/master/server/target/test_params	
	    if( !testName.compare(TRACEROUTE)  || !testName.compare(PIPECHAR) ){
		c_info.setIsSOTest(true);
            	if( !(c_info.getTestSender()).empty() && !(c_info.getTestReceiver()).empty() )
            	    no_defaults = display.getUseDefaults_SO( c_info.getTestSender(), c_info.getTestReceiver() );

            	if( no_defaults > 0 ){
                    display.getTestHosts( c_info );

                    ( c_info.getTestSender() == c_info.getLocalHost() )?
                    	c_info.setIsTestSender(true):
                    	c_info.setIsTestSender(false);
		    //Target of SO Method
                    c_info.setIsTestReceiver(false);
            	}
            	curtest.append( c_info.getTestSender() ); // append server
            	curtest.append( DELIM ); 
            	curtest.append( c_info.getTestReceiver() ); // append client(SR) or target(SO)
            	curtest.append( DELIM );

	 	string params;
		if( no_defaults > 0 ){
                    display.askUserTrace( params );
		    c_info.setTraceParams( params );
		    curtest.append( params );
		}
		else
                    curtest.append( c_info.getTraceParams() );

                display.displayCurrentTest_SO( curtest );

	    } // SR: testname/master/server/recv/test_params
	    else{  
		c_info.setIsSOTest(false);
		if( !(c_info.getTestSender()).empty() && !(c_info.getTestReceiver()).empty() )
		    no_defaults = display.getUseDefaults( c_info.getTestSender(), c_info.getTestReceiver() );

                if( no_defaults > 0 ){
                    display.getTestHosts( c_info );

		    ( c_info.getTestSender() == c_info.getLocalHost() )?
			c_info.setIsTestSender(true):
		    	c_info.setIsTestSender(false);

		    ( c_info.getTestReceiver() == c_info.getLocalHost() )?
			c_info.setIsTestReceiver(true):
			c_info.setIsTestReceiver(false);
	    	}

                // Message Always begin with : testname/master/ 
            	curtest.append( c_info.getTestSender() ); // append server
                curtest.append( DELIM );
            	curtest.append( c_info.getTestReceiver() );
            	curtest.append( DELIM );

	 	string params;
	    	if( testName == IPERF ){
            	    if( no_defaults > 0 ){
                    	display.askUserIperf( params );
		    	c_info.setIperfParams( params );
                    	curtest.append( params );
		    }
		    else
                   	curtest.append( c_info.getIperfParams() );
	    	}
	    	else if( testName == UDPPING ){
            	    if( no_defaults > 0 ){
                        display.askUserPing( params );
		    	c_info.setPingParams( params );
                    	curtest.append( params );
		    }
		    else
                   	curtest.append( c_info.getPingParams() );
	    	}
                else if( testName == MULTIPING ){
            	    if( no_defaults > 0 ){
                    	if( display.askUserMulti(params) < 0 ) return -1;
		    	c_info.setMultiParams( params );
                    	curtest.append( params );
		    }
		    else
                   	curtest.append( c_info.getMultiParams() );
		}
                else if( testName == TUNEDTCP ){
            	    if( no_defaults > 0 ){
                    	display.askUserTunedTcp( params );
		    	c_info.setTunedParams( params );
                    	curtest.append( params );
		    }
		    else
                    	curtest.append( c_info.getTunedParams() );
	 	}
                display.displayCurrentTest( curtest );
            }
        }
	else if( testName == QUIT ){
	    cout << "\nQuit\n";
	    exit(1);
	}
	else{
	    cout << "What Test? " << curtest << endl;
	    fail = 1;
	}
    } while( fail || display.askContinue() < 0 );

    return 1;
}


/* ---------------------------------------------------------------------------------
 * handleRequest_Master
 *   If sender and receiver are the same only make one connection.
 *
 *  const string  curtest
 *  CoordInfo     c_info - Completely Filled in.
 *  NetworkTCP&   network_securetcp
 *  SecureTcp*    relaycoordcon 
 *
 * --------------------------------------------------------------------------------- */

// Master && SERVER && ~REQUESTER

int
handleRequest_Master( const string curtest, CoordInfo c_info, NetworkTCP &network_securetcp, SecureTcp* relaycoordcon   )
{
    bool         fin(true);

    if( !c_info.getIsTestRequester() )
    	c_info.setRequesterName( relaycoordcon->getRemoteName() );

    // Sender Only Test
    if( c_info.getIsSOTest() ){
        debug.printDebug(4, "Sender Only Test\n");

	if( c_info.getIsTestSender() )
            doTest( c_info );
        else{
       	    SecureTcp *testS_cc;
            testS_cc = network_securetcp.newTCP_connect( c_info.getTestSender(), *passwdC_cb_Wrap );
	    coordConSend( testS_cc, curtest );
	    if( wait_for_completed_Test( testS_cc ) < 0 ){
		fin = false;
		cout << "Server Report failed" << endl;
	    }
	    delete testS_cc;
	}
    }
    else if( c_info.getTestSender() != c_info.getTestReceiver() ){
       	SecureTcp *testS_cc = NULL;
       	SecureTcp *testC_cc = NULL;

        if( !c_info.getIsTestReceiver() ){
            testC_cc = network_securetcp.newTCP_connect( c_info.getTestReceiver(), *passwdC_cb_Wrap );
            (testC_cc==NULL)? fin = false: fin;
        }

        if( !c_info.getIsTestSender() ){
            testS_cc = network_securetcp.newTCP_connect( c_info.getTestSender(), *passwdC_cb_Wrap );
	    (testS_cc==NULL)? fin = false: fin;
	}

	if( testC_cc != NULL )
                coordConSend( testC_cc, curtest );
	if( testS_cc != NULL )
                coordConSend( testS_cc, curtest );

        if( fin ){
	    if( c_info.getIsTestReceiver() || c_info.getIsTestSender() )
		    doTest( c_info );

            if( !c_info.getIsTestSender() ){ 
	    	if( wait_for_completed_Test(testS_cc) < 0 ){  // Master waits for test sender
	    	    cout << "Sender Report failed" << endl;
	 	    fin = false;
            	}
		delete testS_cc;
             }
            if( !c_info.getIsTestReceiver() ){
	    	if( wait_for_completed_Test(testC_cc)<0 ){  // Master waits for test receiver
	    	    cout << "Receiver Report failed" << endl;
	 	    fin = false;
                }
	        delete testC_cc;
            }
        }
    }
    else{
	cout << "SR Method: Sender and Receiver must be different\n";
	fin = false;
    }

    if( !c_info.getIsTestRequester() )
    	completed_Test( fin, relaycoordcon ); // tell Requester Done

    return ((fin)? 0:-1);
}


/* ---------------------------------------------------------------------------------
 * wait_for_completed_Test
 *
 *  SecureTcp* testcc
 *
 * --------------------------------------------------------------------------------- */

int
wait_for_completed_Test( SecureTcp* testcc )
{
    debug.printDebug(d2, "wait_for_completed_Test: waiting\n");

    string mess = coordConReceive( testcc );

    if( mess == FINISHED )
	cout << "\nTest: Message received   FINISHED\n";
    else if( mess == FAILED ){
	cout << "Test: Message received   FAILED\n";
	return -1;
    }
    else{
	cout << "wait_for_completed_Test: Message received ?? " << mess << endl;
	return -1;
    }
    return 0;	
}


/* ---------------------------------------------------------------------------------
 * completed_Test( 
 *
 *  bool          isXX
 *  SecureTcp*    tempcc
 *
 * --------------------------------------------------------------------------------- */

void 
completed_Test( bool isXX, SecureTcp* tempcc )
{
    ( isXX )? coordConSend( tempcc, FINISHED ): 
              coordConSend( tempcc, FAILED ); 
}


/*----------------------------------------------------------------------
 * coordConReceive
 *
 *----------------------------------------------------------------------*/

string coordConReceive( SecureTcp *con )
{
    debug.printDebug(d2, "coordconReceive \n");
    char *buf;

    buf = (char *)calloc( 4096, sizeof( char ));
    memset( buf, 0, sizeof(buf) );

    con->readn( buf, 4096 );
    buf[4095] = '\0';
    string msg = buf;
    free( buf );
    debug.printDebug(d2, "coordconReceive: ");
    debug.printDebug(d2, "coordconReceive: %s\n", msg.c_str());

    return msg;
}


/*----------------------------------------------------------------------
 * coordConSend
 *
 *----------------------------------------------------------------------*/

int coordConSend( SecureTcp *con, const string msg )
{
    debug.printDebug(d2, "coordconSend msg= %s\n", msg.c_str());
    char buf[4096];
    int len;

    memset( buf, 0, sizeof(buf) );
    strcpy( buf, msg.c_str() );
    len = strlen( buf );
    buf[len] = '\0';

    con->writen( buf, len+1 );

    return 0;
}


/* -----------------------------------------------------------------
 * doTest:  Launch appropriate Test, MUST be test client or server to
 *		enter this method
 *
 *  SR - getTestSender = sender
 *       getTestReceiver = receiver
 * ----------------------------------------------------------------- */

void
doTest( CoordInfo coordinfo )
{
    debug.printDebug(d2, "doTest\n");
    if( coordinfo.getIsTestSender() && coordinfo.getIsTestReceiver() ){
        debug.printDebug(d2, "doTest ERROR\n");
    }
    else
        tModule.startTest( coordinfo );

    return;
}


/*----------------------------------------------------------------------
 *  sighandler:
 *  Since sigwait() is broken for freeBSD, I am using this routine to
 *  catch and process interrupts.  Solaris 2.6 and freeBSD (2.2.6,
 *  2.2.7) are both happy.
 *----------------------------------------------------------------------*/

void 
sighandler(int sig)
{
   if ( debug.getDebugLevel() > 0)
   {
        cout << "NETTEST: SIGNAL " << sig;
        switch( sig )
        {
            case SIGKILL:
                cout << "  SIGKILL \n";
                break;
            case SIGBUS:
                cout << "  SIGBUS \n";
                break;
            case SIGSEGV:
                cout << "  SIGSEGV \n";
                break;
            case SIGPIPE:
                cout << "  SIGPIPE \n";
                break;
            case SIGHUP:
                cout << "  SIGALRM \n";
                break;
            case SIGQUIT:
                cout << "  SIGALRM \n";
                break;
            case SIGALRM:
                cout << "  SIGALRM \n";
                break;
            default:
                cout << endl;
                break;
        }
        cout << "NETTEST: SHUTTING DOWN ALL TESTS...THIS MAY TAKE A WHILE\n";
   }
   _exit(1);
}
