/* -*-C++-*- */
/*
 *  Copyright (c) 1999 Regents of the University of California.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. All advertising materials mentioning features or use of this software
 *     must display the following acknowledgment:
 *       This product includes software developed by the Imaging and
 *       Distributed Collaboratories Group at Lawrence Berkeley
 *       National Laboratory.
 *  4. Neither the name of the University nor of the Laboratory may be used
 *     to endorse or promote products derived from this software without
 *     specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 */

#ifndef PINGTESTINFO_H
#define PINGTESTINFO_H

#include "TestInfo.h"
#include <string>
#include <iosfwd>

using std::string;

class PingTestInfo : public TestInfo
{
 public:

    PingTestInfo();
    PingTestInfo( const TestInfo& curinfo );
    virtual ~PingTestInfo();
    inline void setMcastTTL( int ttl )
    {
	mMcastTtl = ttl; 
    };
    inline void setSendRate( double rate ) 
    { 
	mSendRate = rate; 
    };
    inline void setMcastAddr( const string& addr )
    {
	mMcastAddr = addr;
    };
    inline void setIsMulti( bool isM )
    {
        isMulti = isM;
    };
    inline int getMcastTTL() const 
    {
	return mMcastTtl; 
    };
    inline double getSendRate() const 
    {
	return mSendRate; 
    };
    inline const string &getMcastAddr() const 
    {
	return mMcastAddr; 
    };
    inline bool  getIsMulti() const
    {
	return isMulti;
    };

    virtual void          setUserParams( void );
    virtual std::ostream& printVariables(std::ostream& stream);

  private:

    bool        isMulti;
    int         mMcastTtl;     // time-to-live to set in packets
    double      mSendRate;  // rate at which to send test packets (number per second)
    string      mMcastAddr;  // multicast address to use for test
};

#endif //PINGTESTINFO_H
